@@ -20,12 +20,14 @@
 //! 2.0 license.
 
 #![cfg_attr(not(feature = "std"), no_std)]
-#![cfg_attr(not(feature = "std"), feature(alloc))]
 
-#[cfg(not(feature = "std"))]
+#[cfg(all(test, not(feature = "std")))]
 #[macro_use]
 extern crate alloc;
 
+#[cfg(all(not(test), not(feature = "std")))]
+extern crate alloc;
+
 #[cfg(not(feature = "std"))]
 use alloc::vec::Vec;
 
@@ -39,6 +41,9 @@ pub(crate) mod lib {
 
     #[cfg(not(feature = "std"))]
     pub(crate) use core::*;
+
+    #[cfg(all(test, not(feature = "std")))]
+    pub(crate) use alloc::{borrow, boxed, rc};
 }
 
 use lib::borrow::{Borrow, BorrowMut};
@@ -131,21 +136,20 @@ impl_array! { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 20, 24,
 /// let mut stack_vec = StackVec::<[u8; 8]>::new();
 /// initialize(&mut stack_vec);
 /// ```
-#[deprecated(note = "Use `Extend` and `Deref<[T]>` instead")]
 pub trait VecLike<T>:
-        ops::Index<usize, Output=T> +
-        ops::IndexMut<usize> +
-        ops::Index<ops::Range<usize>, Output=[T]> +
-        ops::IndexMut<ops::Range<usize>> +
-        ops::Index<ops::RangeFrom<usize>, Output=[T]> +
-        ops::IndexMut<ops::RangeFrom<usize>> +
-        ops::Index<ops::RangeTo<usize>, Output=[T]> +
-        ops::IndexMut<ops::RangeTo<usize>> +
-        ops::Index<ops::RangeFull, Output=[T]> +
-        ops::IndexMut<ops::RangeFull> +
-        ops::DerefMut<Target = [T]> +
-        Extend<T> {
-
+    ops::Index<usize, Output = T>
+    + ops::IndexMut<usize>
+    + ops::Index<ops::Range<usize>, Output = [T]>
+    + ops::IndexMut<ops::Range<usize>>
+    + ops::Index<ops::RangeFrom<usize>, Output = [T]>
+    + ops::IndexMut<ops::RangeFrom<usize>>
+    + ops::Index<ops::RangeTo<usize>, Output = [T]>
+    + ops::IndexMut<ops::RangeTo<usize>>
+    + ops::Index<ops::RangeFull, Output = [T]>
+    + ops::IndexMut<ops::RangeFull>
+    + ops::DerefMut<Target = [T]>
+    + Extend<T>
+{
     /// Append an element to the vector.
     fn push(&mut self, value: T);
 
@@ -209,12 +213,14 @@ pub struct Drain<'a, T: 'a> {
     iter: slice::IterMut<'a, T>,
 }
 
-impl<'a, T: 'a> Iterator for Drain<'a,T> {
+impl<'a, T: 'a> Iterator for Drain<'a, T> {
     type Item = T;
 
     #[inline]
     fn next(&mut self) -> Option<T> {
-        self.iter.next().map(|reference| unsafe { ptr::read(reference) })
+        self.iter
+            .next()
+            .map(|reference| unsafe { ptr::read(reference) })
     }
 
     #[inline]
@@ -226,13 +232,15 @@ impl<'a, T: 'a> Iterator for Drain<'a,T> {
 impl<'a, T: 'a> DoubleEndedIterator for Drain<'a, T> {
     #[inline]
     fn next_back(&mut self) -> Option<T> {
-        self.iter.next_back().map(|reference| unsafe { ptr::read(reference) })
+        self.iter
+            .next_back()
+            .map(|reference| unsafe { ptr::read(reference) })
     }
 }
 
-impl<'a, T> ExactSizeIterator for Drain<'a, T> { }
+impl<'a, T> ExactSizeIterator for Drain<'a, T> {}
 
-impl<'a, T: 'a> Drop for Drain<'a,T> {
+impl<'a, T: 'a> Drop for Drain<'a, T> {
     fn drop(&mut self) {
         // Destroy the remaining elements.
         for _ in self.by_ref() {}
@@ -252,7 +260,10 @@ struct SetLenOnDrop<'a> {
 impl<'a> SetLenOnDrop<'a> {
     #[inline]
     fn new(len: &'a mut usize) -> Self {
-        SetLenOnDrop { local_len: *len, len: len }
+        SetLenOnDrop {
+            local_len: *len,
+            len: len,
+        }
     }
 
     #[inline]
@@ -268,6 +279,24 @@ impl<'a> Drop for SetLenOnDrop<'a> {
     }
 }
 
+struct DropOnPanic<T> {
+    start: *mut T,
+    skip: ops::Range<usize>,
+    len: usize,
+}
+
+impl<T> Drop for DropOnPanic<T> {
+    fn drop(&mut self) {
+        for i in 0..self.len {
+            if !self.skip.contains(&i) {
+                unsafe {
+                    ptr::drop_in_place(self.start.add(i));
+                }
+            }
+        }
+    }
+}
+
 // STACKVEC
 
 /// A `Vec`-like container that stores elements on the stack.
@@ -293,19 +322,17 @@ pub struct StackVec<A: Array> {
     // The capacity field is used for iteration and other optimizations.
     // Publicly expose the fields, so they may be used in constant
     // initialization.
+    pub data: mem::MaybeUninit<A>,
     pub length: usize,
-    pub data: mem::ManuallyDrop<A>,
 }
 
 impl<A: Array> StackVec<A> {
     /// Construct an empty vector
     #[inline]
     pub fn new() -> StackVec<A> {
-        unsafe {
-            StackVec {
-                length: 0,
-                data: mem::uninitialized(),
-            }
+        StackVec {
+            length: 0,
+            data: mem::MaybeUninit::uninit(),
         }
     }
 
@@ -328,6 +355,7 @@ impl<A: Array> StackVec<A> {
     }
 
     /// Construct a new `StackVec` from a `Vec<A::Item>` without bounds checking.
+    #[allow(deprecated)]
     pub unsafe fn from_vec_unchecked(mut vec: Vec<A::Item>) -> StackVec<A> {
         let mut data: A = mem::uninitialized();
         let len = vec.len();
@@ -336,7 +364,7 @@ impl<A: Array> StackVec<A> {
 
         StackVec {
             length: len,
-            data: mem::ManuallyDrop::new(data),
+            data: mem::MaybeUninit::new(data),
         }
     }
 
@@ -355,7 +383,7 @@ impl<A: Array> StackVec<A> {
     pub fn from_buf(buf: A) -> StackVec<A> {
         StackVec {
             length: A::size(),
-            data: mem::ManuallyDrop::new(buf),
+            data: mem::MaybeUninit::new(buf),
         }
     }
 
@@ -395,7 +423,7 @@ impl<A: Array> StackVec<A> {
     pub unsafe fn from_buf_and_len_unchecked(buf: A, len: usize) -> StackVec<A> {
         StackVec {
             length: len,
-            data: mem::ManuallyDrop::new(buf),
+            data: mem::MaybeUninit::new(buf),
         }
     }
 
@@ -456,7 +484,7 @@ impl<A: Array> StackVec<A> {
             if self.len() == 0 {
                 None
             } else {
-                self.length -=1;
+                self.length -= 1;
                 Some(ptr::read(self.as_mut_ptr().padd(self.length)))
             }
         }
@@ -540,36 +568,69 @@ impl<A: Array> StackVec<A> {
 
     /// Insert multiple elements at position `index`, shifting all following elements toward the
     /// back.
-    pub fn insert_many<I: iter::IntoIterator<Item=A::Item>>(&mut self, index: usize, iterable: I) {
-        assert!(index <= self.len());
-
-        let iter = iterable.into_iter();
-        let (lower_bound, upper_bound) = iter.size_hint();
-        let upper_bound = upper_bound.expect("iterable must provide upper bound.");
-        assert!(self.len() + upper_bound <= self.capacity());
-
+    pub fn insert_many<I: iter::IntoIterator<Item = A::Item>>(
+        &mut self,
+        index: usize,
+        iterable: I,
+    ) {
+        let mut iter = iterable.into_iter();
         if index == self.len() {
             return self.extend(iter);
         }
 
+        let (lower_size_bound, _) = iter.size_hint();
+        assert!(lower_size_bound <= lib::isize::MAX as usize); // Ensure offset is indexable
+        assert!(index + lower_size_bound >= index); // Protect against overflow
+        assert!(self.len() + lower_size_bound <= self.capacity());
+
+        let mut num_added = 0;
+        let old_len = self.len();
+        assert!(index <= old_len);
+
+
         unsafe {
-            let old_len = self.len();
-            let ptr = self.as_mut_ptr().padd(index);
+            let start = self.as_mut_ptr();
+            let ptr = start.add(index);
 
             // Move the trailing elements.
-            ptr::copy(ptr, ptr.padd(lower_bound), old_len - index);
+            ptr::copy(ptr, ptr.padd(lower_size_bound), old_len - index);
 
             // In case the iterator panics, don't double-drop the items we just copied above.
-            self.set_len(index);
+            self.set_len(0);
+            let mut guard = DropOnPanic {
+                start,
+                skip: index..(index + lower_size_bound),
+                len: old_len + lower_size_bound,
+            };
 
-            let mut num_added = 0;
-            for element in iter {
-                let cur = ptr.padd(num_added);
+            while num_added < lower_size_bound {
+                let element = match iter.next() {
+                    Some(x) => x,
+                    None => break,
+                };
+                let cur = ptr.add(num_added);
                 ptr::write(cur, element);
+                guard.skip.start += 1;
                 num_added += 1;
             }
 
+            if num_added < lower_size_bound {
+                // Iterator provided fewer elements than the hint. Move the tail backward.
+                ptr::copy(
+                    ptr.add(lower_size_bound),
+                    ptr.add(num_added),
+                    old_len - index,
+                );
+            }
+            // There are no more duplicate or uninitialized slots, so the guard is not needed.
             self.set_len(old_len + num_added);
+            mem::forget(guard);
+
+            // Insert any remaining elements one-by-one.
+            for element in iter {
+                self.insert(index + num_added, element);
+                num_added += 1;
+            }
         }
     }
 
@@ -584,9 +645,9 @@ impl<A: Array> StackVec<A> {
             Err(self)
         } else {
             unsafe {
-                let data = ptr::read(&self.data);
-                mem::forget(self);
-                Ok(mem::ManuallyDrop::into_inner(data))
+                let this = mem::ManuallyDrop::new(self);
+                let array = ptr::read(this.as_ptr() as *const A);
+                Ok(array)
             }
         }
     }
@@ -610,13 +671,17 @@ impl<A: Array> StackVec<A> {
     }
 
     /// Removes consecutive duplicate elements.
-    pub fn dedup(&mut self) where A::Item: PartialEq<A::Item> {
+    pub fn dedup(&mut self)
+    where
+        A::Item: PartialEq<A::Item>,
+    {
         self.dedup_by(|a, b| a == b);
     }
 
     /// Removes consecutive duplicate elements using the given equality relation.
     pub fn dedup_by<F>(&mut self, mut same_bucket: F)
-        where F: FnMut(&mut A::Item, &mut A::Item) -> bool
+    where
+        F: FnMut(&mut A::Item, &mut A::Item) -> bool,
     {
         // See the implementation of Vec::dedup_by in the
         // standard library for an explanation of this algorithm.
@@ -647,14 +712,18 @@ impl<A: Array> StackVec<A> {
 
     /// Removes consecutive elements that map to the same key.
     pub fn dedup_by_key<F, K>(&mut self, mut key: F)
-        where F: FnMut(&mut A::Item) -> K,
-              K: PartialEq<K>
+    where
+        F: FnMut(&mut A::Item) -> K,
+        K: PartialEq<K>,
     {
         self.dedup_by(|a, b| key(a) == key(b));
     }
 }
 
-impl<A: Array> StackVec<A> where A::Item: Copy {
+impl<A: Array> StackVec<A>
+where
+    A::Item: Copy,
+{
     /// Copy the elements from a slice into a new `StackVec`.
     ///
     /// For slices of `Copy` types, this is more efficient than `StackVec::from(slice)`.
@@ -663,10 +732,11 @@ impl<A: Array> StackVec<A> where A::Item: Copy {
         StackVec {
             length: slice.len(),
             data: unsafe {
-                let mut data: A = mem::uninitialized();
-                ptr::copy_nonoverlapping(slice.as_ptr(), data.ptr_mut(), slice.len());
-                mem::ManuallyDrop::new(data)
-            }
+                let mut data: mem::MaybeUninit<A> = mem::MaybeUninit::uninit();
+                let ptr = data.as_mut_ptr() as *mut A::Item;
+                ptr::copy_nonoverlapping(slice.as_ptr(), ptr, slice.len());
+                data
+            },
         }
     }
 
@@ -696,7 +766,10 @@ impl<A: Array> StackVec<A> where A::Item: Copy {
     }
 }
 
-impl<A: Array> StackVec<A> where A::Item: Clone {
+impl<A: Array> StackVec<A>
+where
+    A::Item: Clone,
+{
     /// Resizes the vector so that its length is equal to `len`.
     ///
     /// If `len` is less than the current length, the vector simply truncated.
@@ -740,7 +813,8 @@ impl<A: Array> ops::Deref for StackVec<A> {
     #[inline]
     fn deref(&self) -> &[A::Item] {
         unsafe {
-            slice::from_raw_parts(self.data.ptr(), self.len())
+            let ptr = self.data.as_ptr() as *const A::Item;
+            slice::from_raw_parts(ptr, self.len())
         }
     }
 }
@@ -749,7 +823,8 @@ impl<A: Array> ops::DerefMut for StackVec<A> {
     #[inline]
     fn deref_mut(&mut self) -> &mut [A::Item] {
         unsafe {
-            slice::from_raw_parts_mut(self.data.ptr_mut(), self.len())
+            let ptr = self.data.as_mut_ptr() as *mut A::Item;
+            slice::from_raw_parts_mut(ptr, self.len())
         }
     }
 }
@@ -802,7 +877,10 @@ impl<A: Array<Item = u8>> io::Write for StackVec<A> {
     }
 }
 
-impl<'a, A: Array> From<&'a [A::Item]> for StackVec<A> where A::Item: Clone {
+impl<'a, A: Array> From<&'a [A::Item]> for StackVec<A>
+where
+    A::Item: Clone,
+{
     #[inline]
     fn from(slice: &'a [A::Item]) -> StackVec<A> {
         slice.into_iter().cloned().collect()
@@ -839,7 +917,7 @@ macro_rules! impl_index {
                 &mut (&mut **self)[index]
             }
         }
-    }
+    };
 }
 
 impl_index!(usize, A::Item);
@@ -854,13 +932,15 @@ impl_index!(ops::RangeInclusive<usize>, [A::Item]);
 #[cfg(has_range_inclusive)]
 impl_index!(ops::RangeToInclusive<usize>, [A::Item]);
 
-impl<A: Array> ExtendFromSlice<A::Item> for StackVec<A> where A::Item: Copy {
+impl<A: Array> ExtendFromSlice<A::Item> for StackVec<A>
+where
+    A::Item: Copy,
+{
     fn extend_from_slice(&mut self, other: &[A::Item]) {
         StackVec::extend_from_slice(self, other)
     }
 }
 
-#[allow(deprecated)]
 impl<A: Array> VecLike<A::Item> for StackVec<A> {
     #[inline]
     fn push(&mut self, value: A::Item) {
@@ -874,7 +954,7 @@ impl<A: Array> VecLike<A::Item> for StackVec<A> {
 }
 
 impl<A: Array> iter::FromIterator<A::Item> for StackVec<A> {
-    fn from_iter<I: iter::IntoIterator<Item=A::Item>>(iterable: I) -> StackVec<A> {
+    fn from_iter<I: iter::IntoIterator<Item = A::Item>>(iterable: I) -> StackVec<A> {
         let mut v = StackVec::new();
         v.extend(iterable);
         v
@@ -882,34 +962,20 @@ impl<A: Array> iter::FromIterator<A::Item> for StackVec<A> {
 }
 
 impl<A: Array> Extend<A::Item> for StackVec<A> {
-    fn extend<I: iter::IntoIterator<Item=A::Item>>(&mut self, iterable: I) {
-        let mut iter = iterable.into_iter();
-        let (lower_bound, upper_bound) = iter.size_hint();
-        let upper_bound = upper_bound.expect("iterable must provide upper bound.");
-        assert!(self.len() + upper_bound <= self.capacity());
-
-        unsafe {
-            let len = self.len();
-            let ptr = self.as_mut_ptr().padd(len);
-            let mut count = 0;
-            while count < lower_bound {
-                if let Some(out) = iter.next() {
-                    ptr::write(ptr.padd(count), out);
-                    count += 1;
-                } else {
-                    break;
-                }
-            }
-            self.set_len(len + count);
-        }
-
-        for elem in iter {
+    fn extend<I: iter::IntoIterator<Item = A::Item>>(&mut self, iterable: I) {
+        // size_hint() has no safety guarantees, and TrustedLen
+        // is nightly only, so we can't do any optimizations with
+        // size_hint.
+        for elem in iterable.into_iter() {
             self.push(elem);
         }
     }
 }
 
-impl<A: Array> fmt::Debug for StackVec<A> where A::Item: fmt::Debug {
+impl<A: Array> fmt::Debug for StackVec<A>
+where
+    A::Item: fmt::Debug,
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_list().entries(self.iter()).finish()
     }
@@ -930,7 +996,10 @@ impl<A: Array> Drop for StackVec<A> {
     }
 }
 
-impl<A: Array> Clone for StackVec<A> where A::Item: Clone {
+impl<A: Array> Clone for StackVec<A>
+where
+    A::Item: Clone,
+{
     fn clone(&self) -> StackVec<A> {
         let mut new_vector = StackVec::new();
         for element in self.iter() {
@@ -941,7 +1010,8 @@ impl<A: Array> Clone for StackVec<A> where A::Item: Clone {
 }
 
 impl<A: Array, B: Array> PartialEq<StackVec<B>> for StackVec<A>
-    where A::Item: PartialEq<B::Item>
+where
+    A::Item: PartialEq<B::Item>,
 {
     #[inline]
     fn eq(&self, other: &StackVec<B>) -> bool {
@@ -954,31 +1024,38 @@ impl<A: Array, B: Array> PartialEq<StackVec<B>> for StackVec<A>
     }
 }
 
-impl<A: Array> Eq for StackVec<A> where A::Item: Eq {
-}
+impl<A: Array> Eq for StackVec<A> where A::Item: Eq {}
 
-impl<A: Array> PartialOrd for StackVec<A> where A::Item: PartialOrd {
+impl<A: Array> PartialOrd for StackVec<A>
+where
+    A::Item: PartialOrd,
+{
     #[inline]
     fn partial_cmp(&self, other: &StackVec<A>) -> Option<cmp::Ordering> {
         PartialOrd::partial_cmp(&**self, &**other)
     }
 }
 
-impl<A: Array> Ord for StackVec<A> where A::Item: Ord {
+impl<A: Array> Ord for StackVec<A>
+where
+    A::Item: Ord,
+{
     #[inline]
     fn cmp(&self, other: &StackVec<A>) -> cmp::Ordering {
         Ord::cmp(&**self, &**other)
     }
 }
 
-impl<A: Array> hash::Hash for StackVec<A> where A::Item: hash::Hash {
+impl<A: Array> hash::Hash for StackVec<A>
+where
+    A::Item: hash::Hash,
+{
     fn hash<H: hash::Hasher>(&self, state: &mut H) {
         (**self).hash(state)
     }
 }
 
-unsafe impl<A: Array> Send for StackVec<A> where A::Item: Send {
-}
+unsafe impl<A: Array> Send for StackVec<A> where A::Item: Send {}
 
 /// An iterator that consumes a `StackVec` and yields its items by value.
 ///
@@ -993,8 +1070,7 @@ pub struct IntoIter<A: Array> {
 
 impl<A: Array> Drop for IntoIter<A> {
     fn drop(&mut self) {
-        for _ in self {
-        }
+        for _ in self {}
     }
 }
 
@@ -1005,8 +1081,7 @@ impl<A: Array> Iterator for IntoIter<A> {
     fn next(&mut self) -> Option<A::Item> {
         if self.current == self.end {
             None
-        }
-        else {
+        } else {
             unsafe {
                 let current = self.current;
                 self.current += 1;
@@ -1027,8 +1102,7 @@ impl<A: Array> DoubleEndedIterator for IntoIter<A> {
     fn next_back(&mut self) -> Option<A::Item> {
         if self.current == self.end {
             None
-        }
-        else {
+        } else {
             unsafe {
                 self.end -= 1;
                 Some(ptr::read(self.data.as_ptr().padd(self.end)))
@@ -1037,8 +1111,7 @@ impl<A: Array> DoubleEndedIterator for IntoIter<A> {
     }
 }
 
-impl<A: Array> ExactSizeIterator for IntoIter<A> {
-}
+impl<A: Array> ExactSizeIterator for IntoIter<A> {}
 
 impl<A: Array> IntoIterator for StackVec<A> {
     type IntoIter = IntoIter<A>;
@@ -1137,8 +1210,91 @@ macro_rules! stackvec {
 #[cfg(test)]
 mod test {
     use super::*;
-    use super::lib::iter::FromIterator;
-    use super::lib::rc::Rc;
+    use crate::lib::borrow::ToOwned;
+    use crate::lib::boxed::Box;
+    use crate::lib::iter::FromIterator;
+    use crate::lib::rc::Rc;
+
+    struct BadBoundsIterator1(u8);
+
+    impl BadBoundsIterator1 {
+        pub fn new() -> Self {
+            BadBoundsIterator1(0)
+        }
+    }
+
+    impl Iterator for BadBoundsIterator1 {
+        type Item = u8;
+
+        fn next(&mut self) -> Option<Self::Item> {
+            self.0 += 1;
+            if self.0 >= 10 {
+                None
+            } else {
+                Some(0x41)
+            }
+        }
+
+        fn size_hint(&self) -> (usize, Option<usize>) {
+            let lower_bound = 20;
+            let upper_bound = Some(0);
+            (lower_bound, upper_bound)
+        }
+    }
+
+    struct BadBoundsIterator2(u8);
+
+    impl BadBoundsIterator2 {
+        pub fn new() -> Self {
+            BadBoundsIterator2(0)
+        }
+    }
+
+    impl Iterator for BadBoundsIterator2 {
+        type Item = u8;
+
+        fn next(&mut self) -> Option<Self::Item> {
+            self.0 += 1;
+            if self.0 >= 30 {
+                None
+            } else {
+                Some(0x41)
+            }
+        }
+
+        fn size_hint(&self) -> (usize, Option<usize>) {
+            let lower_bound = 0;
+            let upper_bound = Some(0);
+            (lower_bound, upper_bound)
+        }
+    }
+
+    struct BadSizeHint(u8);
+
+    impl BadSizeHint {
+        pub fn new(start: u8) -> Self {
+            BadSizeHint(start)
+        }
+    }
+
+    impl Iterator for BadSizeHint {
+        type Item = u8;
+
+        fn next(&mut self) -> Option<Self::Item> {
+            self.0 += 1;
+            if self.0 >= 30 {
+                None
+            } else {
+                Some(0x41)
+            }
+        }
+
+        fn size_hint(&self) -> (usize, Option<usize>) {
+            let lower_bound = 0;
+            let upper_bound = None;
+            (lower_bound, upper_bound)
+        }
+    }
 
     #[test]
     pub fn test_zero() {
@@ -1160,10 +1316,7 @@ mod test {
         let mut v = StackVec::<[_; 16]>::new();
         v.push("hello".to_owned());
         v.push("there".to_owned());
-        assert_eq!(&*v, &[
-            "hello".to_owned(),
-            "there".to_owned(),
-        ][..]);
+        assert_eq!(&*v, &["hello".to_owned(), "there".to_owned(),][..]);
     }
 
     #[test]
@@ -1176,12 +1329,15 @@ mod test {
         v.push("burma".to_owned());
         assert_eq!(v[0], "hello");
         v.push("shave".to_owned());
-        assert_eq!(&*v, &[
-            "hello".to_owned(),
-            "there".to_owned(),
-            "burma".to_owned(),
-            "shave".to_owned(),
-        ][..]);
+        assert_eq!(
+            &*v,
+            &[
+                "hello".to_owned(),
+                "there".to_owned(),
+                "burma".to_owned(),
+                "shave".to_owned(),
+            ][..]
+        );
     }
 
     #[test]
@@ -1196,16 +1352,19 @@ mod test {
         v.push("there".to_owned());
         v.push("burma".to_owned());
         v.push("shave".to_owned());
-        assert_eq!(&*v, &[
-            "hello".to_owned(),
-            "there".to_owned(),
-            "burma".to_owned(),
-            "shave".to_owned(),
-            "hello".to_owned(),
-            "there".to_owned(),
-            "burma".to_owned(),
-            "shave".to_owned(),
-        ][..]);
+        assert_eq!(
+            &*v,
+            &[
+                "hello".to_owned(),
+                "there".to_owned(),
+                "burma".to_owned(),
+                "shave".to_owned(),
+                "hello".to_owned(),
+                "there".to_owned(),
+                "burma".to_owned(),
+                "shave".to_owned(),
+            ][..]
+        );
     }
 
     /// https://github.com/servo/rust-smallvec/issues/4
@@ -1310,7 +1469,47 @@ mod test {
         }
         assert_eq!(v.len(), 4);
         v.insert_many(1, [5, 6].iter().cloned());
-        assert_eq!(&v.iter().map(|v| *v).collect::<Vec<_>>(), &[0, 5, 6, 1, 2, 3]);
+        assert_eq!(
+            &v.iter().map(|v| *v).collect::<Vec<_>>(),
+            &[0, 5, 6, 1, 2, 3]
+        );
+    }
+
+    #[test]
+    fn test_insert_many_buggy_iterator() {
+        let mut v: StackVec<[u8; 64]> = StackVec::new();
+        for x in 0..4 {
+            v.push(x);
+        }
+        v.insert_many(1, BadBoundsIterator1::new());
+        assert_eq!(
+            &v.iter().map(|v| *v).collect::<Vec<_>>(),
+            &[0, 65, 65, 65, 65, 65, 65, 65, 65, 65, 1, 2, 3]
+        );
+
+        let mut v: StackVec<[u8; 64]> = StackVec::new();
+        for x in 0..4 {
+            v.push(x);
+        }
+        v.insert_many(1, BadBoundsIterator2::new());
+        assert_eq!(v.len(), 33);
+
+        let mut v: StackVec<[u8; 64]> = StackVec::new();
+        for x in 0..4 {
+            v.push(x);
+        }
+        v.insert_many(1, BadSizeHint::new(1));
+        assert_eq!(v.len(), 32);
+    }
+
+    #[should_panic]
+    #[test]
+    fn test_insert_many_panic_buggy_iterator() {
+        let mut v: StackVec<[u8; 8]> = StackVec::new();
+        for x in 0..4 {
+            v.push(x);
+        }
+        v.insert_many(1, BadBoundsIterator2::new());
     }
 
     #[test]
@@ -1321,7 +1520,10 @@ mod test {
         }
         assert_eq!(v.len(), 4);
         v.insert_from_slice(1, &[5, 6]);
-        assert_eq!(&v.iter().map(|v| *v).collect::<Vec<_>>(), &[0, 5, 6, 1, 2, 3]);
+        assert_eq!(
+            &v.iter().map(|v| *v).collect::<Vec<_>>(),
+            &[0, 5, 6, 1, 2, 3]
+        );
     }
 
     #[test]
@@ -1332,7 +1534,10 @@ mod test {
         }
         assert_eq!(v.len(), 4);
         v.extend_from_slice(&[5, 6]);
-        assert_eq!(&v.iter().map(|v| *v).collect::<Vec<_>>(), &[0, 1, 2, 3, 5, 6]);
+        assert_eq!(
+            &v.iter().map(|v| *v).collect::<Vec<_>>(),
+            &[0, 1, 2, 3, 5, 6]
+        );
     }
 
     #[test]
@@ -1394,8 +1599,8 @@ mod test {
     #[cfg(feature = "std")]
     #[test]
     fn test_hash() {
-        use std::hash::Hash;
         use std::collections::hash_map::DefaultHasher;
+        use std::hash::Hash;
 
         {
             let mut a: StackVec<[u32; 2]> = StackVec::new();
@@ -1439,7 +1644,7 @@ mod test {
 
     #[test]
     fn test_borrow() {
-        use std::borrow::Borrow;
+        use lib::borrow::Borrow;
 
         let mut a: StackVec<[u32; 3]> = StackVec::new();
         a.push(1);
@@ -1452,7 +1657,7 @@ mod test {
 
     #[test]
     fn test_borrow_mut() {
-        use std::borrow::BorrowMut;
+        use lib::borrow::BorrowMut;
 
         let mut a: StackVec<[u32; 3]> = StackVec::new();
         a.push(1);
@@ -1499,7 +1704,10 @@ mod test {
     #[test]
     fn test_from_slice() {
         assert_eq!(&StackVec::<[u32; 2]>::from_slice(&[1][..])[..], [1]);
-        assert_eq!(&StackVec::<[u32; 3]>::from_slice(&[1, 2, 3][..])[..], [1, 2, 3]);
+        assert_eq!(
+            &StackVec::<[u32; 3]>::from_slice(&[1, 2, 3][..])[..],
+            [1, 2, 3]
+        );
     }
 
     #[test]
@@ -1510,7 +1718,6 @@ mod test {
     }
 
     #[test]
-    #[allow(deprecated)]
     fn veclike_deref_slice() {
         use super::VecLike;
 
